/*-
 * Copyright (C) 2010-2012, Centre National de la Recherche Scientifique,
 *                          Institut Polytechnique de Bordeaux,
 *                          Universite Bordeaux 1.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <bfd.h>
#include <dis-asm.h>

#include "decoders/binutils/arm/arm_parser.hh"

#include "arm_decoder.hh"

using namespace std;
using namespace binutils;

bool arm_scanner_open(const string &instr);
void arm_scanner_close();

bool
arm_decoder_func(MicrocodeArchitecture *arch, 
		 Microcode *mc, 
		 const string &instruction,
                 const ConcreteAddress &start,
                 const ConcreteAddress &next)
{
  if (!arm_scanner_open(instruction))
    return false;

  arm::parser_data data;

  data.instruction = instruction;
  data.mc = mc;
  data.start_ma = MicrocodeAddress(start.get_address ());
  data.next_ma = MicrocodeAddress(next.get_address ());
  data.arch = arch;

  arm::parser parser(data);
  bool result = (parser.parse() == 0);
  arm_scanner_close();

  return result;
}
